/* ****************************************************************************
 *  Name:
 *  NetID:
 *  Precept:
 *
 *  Compilation:  javac Grid.java
 *  Execution:    java Grid
 *
 *  Description:  A generic 2D Grid.
 ******************************************************************************/

import edu.princeton.cs.algs4.StdOut;
import edu.princeton.cs.algs4.StdRandom;
import java.util.Iterator;
import java.util.NoSuchElementException;

// A generic 2D grid of elements
public class Grid<Item> implements Iterable<Item> {

    private int n;          // the grid is of size n-by-n
    private Item[][] grid;  // a 2D array for holding items of any type


    // Creates an n-by-n grid
    public Grid(int n) {
        this.n = n;
        grid = (Item[][]) new Object[n][n];
    }


    // Sets the item at (row, col) in the grid to the given item
    public void set(int row, int col, Item item) {
        if (row < 0 || col < 0 || row >= n || col >= n)
            throw new IllegalArgumentException();
        grid[row][col] = item;
    }


    // Returns the item at (row, col) in the grid
    public Item get(int row, int col) {
       if (row < 0 || col < 0 || row >= n || col >= n)
            throw new IllegalArgumentException();
        return grid[row][col];
    }


    // Returns an instance of RowMajorIterator
    public Iterator<Item> iterator() {
        return new GridIterator();
    }


    // Returns an instance of ColMajorIterator
    public Iterator<Item> colMajorIterator() {
        return new ColMajorIterator();
    }


    // Allows iterating over the grid in row-major order
    private class GridIterator implements Iterator<Item> {
        private int i = -1;

        // creates a new iterator
        public GridIterator() {
        }

        // is there a next item in the iterator?
        public boolean hasNext() {
            return i < n * n - 1;
        }

        // this method is optional in Iterator interface
        public void remove() {
            throw new UnsupportedOperationException();
        }

        // returns the next item in the iterator (and advances the iterator)
        public Item next() {
            if (!hasNext()) throw new NoSuchElementException();
            i++;
            return grid[i / n][i % n];
        }
    }


    // Allows iterating over the grid in column-major order
    private class ColMajorIterator implements Iterator<Item> {
        private int j = -1;

        // creates a new iterator
        public ColMajorIterator() {
        }

        // is there a next item in the iterator?
        public boolean hasNext() {
            return j < n * n - 1;
        }

        // this method is optional in Iterator interface
        public void remove() {
            throw new UnsupportedOperationException();
        }

        // returns the next item in the iterator (and advances the iterator)
        public Item next() {
            if (!hasNext()) throw new NoSuchElementException();
            j++;
            return grid[j % n][j / n];
        }
    }

    public static void main(String[] args) {
        int RANGE = 1000;

        // Create a size-by-size grid of Integers
        int size = Integer.parseInt(args[0]);
        Grid<Integer> myGrid = new Grid<Integer>(size);


        // Fill the grid with random numbers
        StdOut.println("2D Grid: ");
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                int rand = StdRandom.uniform(RANGE);
                StdOut.print(rand + "\t");
                myGrid.set(i, j, rand);
            }
            StdOut.println();
        }


        // Print elements in row major order
        StdOut.print("In Row-Major order: ");
        for (int i : myGrid)
            StdOut.print(i + " ");
        StdOut.println();


        // Print elements in column major order
        StdOut.print("In Column-Major order: ");
        Iterator<Integer> iter = myGrid.colMajorIterator();
        while (iter.hasNext())
            StdOut.print(iter.next() + " ");
        StdOut.println();


        // Check if all elements in the grid are distinct.
        // This is going to work only if the Iterator objects
        // can iterate independently.
        boolean distinct = true;
        for (int i : myGrid) {
            int count = 0;
            for (int j : myGrid)
                if (i == j)
                    count++;
            if (count > 1) {
                distinct = false;
                break;
            }
        }

        if (!distinct)
            StdOut.print("Not ");
        StdOut.println("Distinct");
    }
}
